import subprocess
import requests
import re

from rayasdk.constants import *
from rayasdk.logger import *
from rayasdk.messages import *
from rayasdk.utils import download_simulator, simulator_not_found, RayaOS_not_found
from rayasdk.container_handlers.docker_handler import *


def check_simulation_versions():
    vcs = URVCS()
    # If there is a patch update available, update
    if not vcs.patch_updated():
        for raya_component in RAYA_OS_COMPONENTS:
            # Do not check SDK
            if raya_component == 'SDK':
                continue
            if not vcs.patch_updated(raya_component):
                log_info(f'Updating {raya_component}')
                if not vcs.update_patch(raya_component):
                    log_error(f'Could not update {raya_component}')
                    return False
                log_info('Run the command again')
                return False
        log_info("All components are up to date.")
    # If there is a major/minor version let the user know
    if not vcs.full_updated() or not vcs.minor_updated():
        return False
    else:
        return True

def check_sdk_version():
    vcs = URVCS()
    component='SDK'
    if not vcs.full_updated(component=component) or not vcs.minor_updated(component=component):
        latest = vcs.get_latest_version(component=component)
        log_warning((
                'RayaSDK is not up to date. To update run '
                f'\'pip install rayasdk=={latest}\''
            ))
    return True


class URVCS:

    COMMAND = 'update'

    def __init__(self):
        self.current_version = None
        self.host_os = platform.system()

    def init_parser(self, subparser):
        self.parser = subparser.add_parser(
            type(self).COMMAND, help='Update the SDK, Unity or Docker')

    def run(self, args, unknownargs):
        self.args = args
        self.unknownargs = unknownargs
        log_info('Checking for updates...')
        
        if not self.full_updated() or simulator_not_found() or RayaOS_not_found():
            download_input = input(f'Do you want to download the Simulator and RayaOS? [Y/n]')
            if download_input not in ['y', 'Y', 'yes', 'Yes', 'YES', '']:
                return False
        
        if self.update_full():
            log_success('Everything is up to date')
            return True
        else:
            log_error('Update failed.')
            return False

    def __get_major(self, version):
        return int(version.split('.')[0])

    def __get_minor(self, version):
        return int(version.split('.')[1])

    def __get_patch(self, version):
        patch = version.split('.')[2]
        if patch:
            return str(patch)
        return '0'

    def __local_version(self, component):
        '''
        Return the local version of the component
        '''
        if component == 'SDK':
            res = subprocess.check_output(
                "pip show rayasdk --version|grep Version | awk '{print $2}'",
                shell=True)
            return re.findall(r'\d+\.\d+\.\d+', res.decode('utf-8'))[0]
        elif component == 'Unity':
            return GARYSIM_VERSION
        elif component == 'Docker':
            return RAYAENV_DOCKER_VERSION

    def __get_latest_version(self, component):
        '''
        Return the latest version of the component
        '''
        # response = requests.request("GET", f'{VCS_URL}/latest/{component}')
        # return(data['versionId'])
        response = requests.request("GET", f'{VCS_URL}')
        data = response.json()
        for raya_component in data:
            if raya_component['name'] == component:
                return raya_component['versionId']

    def __update_constants_file(self, component):
        new_lines = []
        try:
            with open(f'{UR_HOME}/versions.txt', 'r+') as f:
                for line in f:
                    if re.match(r'Docker:\d.\d.\d',
                                line) and component == 'Docker':
                        new_lines.append(
                            f'Docker:{self.__get_latest_version(component)}\n')
                    elif re.match(r'Unity:\d.\d.\d',
                                  line) and component == 'Unity':
                        new_lines.append(
                            f'Unity:{self.__get_latest_version(component)}')
                    else:
                        new_lines.append(line)
                f.seek(0)
                f.writelines(new_lines)
                f.truncate()
                f.close
                update_constants()
                return True
        except:
            log_error('Error updating the constants file')
            return False

    def full_updated(self, component=None):
        if component:
            local = self.__local_version(component)
            latest = self.__get_latest_version(component)
            diff_major = self.__get_major(latest) > self.__get_major(local)
            if diff_major:
                log_warning((f'{component} have a major update, '
                        f'current: \'{local}\' -> latest: \'{latest}\''))
                return False
            else:
                return True
        
        updated = True
        for raya_component in RAYA_OS_COMPONENTS:
            local = self.__local_version(raya_component)
            latest = self.__get_latest_version(raya_component)
            diff_major = self.__get_major(latest) > self.__get_major(local)
            if diff_major:
                log_warning((f'{raya_component} have a major update, '
                        f'current: \'{local}\' -> latest: \'{latest}\''))
                updated = False
        return updated

    def update_full(self):
        # Update Unity
        if self.__get_latest_version('Unity') != self.__local_version(
                'Unity') or simulator_not_found():
            response = requests.request("GET", VCS_URL)
            if not download_simulator(response=response.json()):
                return False
            self.__update_constants_file('Unity')
            log_success('Simulator updated successfully')

        # Update Docker
        if self.__get_latest_version('Docker') != self.__local_version(
                'Docker') or RayaOS_not_found():
            version = self.__get_latest_version('Docker')
            if not download_raya_image(version):
                return False
            self.__update_constants_file('Docker')
            log_success('Raya_OS updated successfully')

        return True

    def minor_updated(self, component=None):
        '''Function to check if the latest version is a minor version'''
        if component:
            local = self.__local_version(component)
            latest = self.__get_latest_version(component)
            same_major = self.__get_major(latest) == self.__get_major(local)
            diff_minor = self.__get_minor(latest) > self.__get_minor(local)
            if same_major and diff_minor:
                log_warning((f'{component} have a minor update, '
                        f'current: \'{local}\' -> latest: \'{latest}\''))
                return False
            else:
                return True
        
        updated = True
        for raya_component in RAYA_OS_COMPONENTS:
            local = self.__local_version(raya_component)
            latest = self.__get_latest_version(raya_component)
            same_major = self.__get_major(latest) == self.__get_major(local)
            diff_minor = self.__get_minor(latest) > self.__get_minor(local)
            if same_major and diff_minor:
                log_warning((f'{raya_component} have a minor update, '
                        f'current: \'{local}\' -> latest: \'{latest}\''))
                updated = False
        return updated

    def update_minor(self, component):
        download_input = input(f'Do you want to download the Simulator and the Docker Image? [Y/n]\'')
        if download_input not in ['y', 'Y', 'yes', 'Yes', 'YES', '']:
            return False
        
        '''Update the component and return True if the update was successful'''
        if component == 'Unity':
            response = requests.request("GET", VCS_URL)
            version = self.__get_latest_version(component)
            if download_simulator(response=response.json()):
                if self.__update_constants_file(component):
                    return True
        elif component == 'Docker':
            version = self.__get_latest_version(component)
            if download_raya_image(version):
                if self.__update_constants_file(component):
                    return True
        return False

    def patch_updated(self, component=None):
        '''Function to check if the latest version is a patch'''
        if component:
            local = self.__local_version(component)
            latest = self.__get_latest_version(component)
            same_major = self.__get_major(latest) == self.__get_major(local)
            same_minor = self.__get_minor(latest) == self.__get_minor(local)
            diff_patch = self.__get_patch(local) != self.__get_patch(latest)
            if same_major and same_minor and diff_patch:
                log_warning(
                    (f'{component} have an patch update it is going to be'
                     f' updated to version \'{latest}\', current \'{local}\''))
                return False
            else:
                return True

        updated = True
        for raya_component in RAYA_OS_COMPONENTS:
            local = self.__local_version(raya_component)
            latest = self.__get_latest_version(raya_component)
            same_major = self.__get_major(latest) == self.__get_major(local)
            same_minor = self.__get_minor(latest) == self.__get_minor(local)
            diff_patch = self.__get_patch(local) != self.__get_patch(latest)
            if same_major and same_minor and diff_patch:
                log_warning(
                    (f'{raya_component} have an patch update it is going to be'
                     f' updated to version \'{latest}\', current \'{local}\''))
                updated = False
        return updated

    def update_patch(self, component):
        '''Update the component and return True if the update was successful'''
        if component == 'Unity':
            response = requests.request("GET", VCS_URL)
            version = self.__get_latest_version(component)
            if download_simulator(response=response.json()):
                if self.__update_constants_file(component):
                    return True
        elif component == 'Docker':
            version = self.__get_latest_version(component)
            if download_raya_image(version):
                if self.__update_constants_file(component):
                    return True
        return False

    def get_latest_version(self, component):
        return self.__get_latest_version(component=component)